@@ -69,20 +69,10 @@ def draw_page(page, stream):
 
 
 def draw_box_background_and_border(stream, page, box):
-    draw_background(stream, box.background)
     if isinstance(box, boxes.TableBox):
-        draw_table_backgrounds(stream, page, box)
-        if box.style['border_collapse'] == 'separate':
-            draw_border(stream, box)
-            for row_group in box.children:
-                for row in row_group.children:
-                    for cell in row.children:
-                        if (cell.style['empty_cells'] == 'show' or
-                                not cell.empty):
-                            draw_border(stream, cell)
-        else:
-            draw_collapsed_borders(stream, box)
+        draw_table(stream, box)
     else:
+        draw_background(stream, box.background)
         draw_border(stream, box)
 
 
@@ -118,10 +108,7 @@ def draw_stacking_context(stream, stacking_context):
 
         if box.style['opacity'] < 1:
             original_stream = stream
-            stream = stream.add_group([
-                box.border_box_x(), box.border_box_y(),
-                box.border_box_x() + box.border_width(),
-                box.border_box_y() + box.border_height()])
+            stream = stream.add_group(stream.page_rectangle)
 
         if box.transformation_matrix:
             if box.transformation_matrix.determinant:
@@ -135,7 +122,7 @@ def draw_stacking_context(stream, stacking_context):
         if isinstance(box, (boxes.BlockBox, boxes.MarginBox,
                             boxes.InlineBlockBox, boxes.TableCellBox,
                             boxes.FlexContainerBox)):
-            # The canvas background was removed by set_canvas_background
+            # The canvas background was removed by layout_backgrounds
             draw_box_background_and_border(stream, stacking_context.page, box)
 
         with stacked(stream):
@@ -341,23 +328,6 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
             draw_background_image(stream, layer, bg.image_rendering)
 
 
-def draw_table_backgrounds(stream, page, table):
-    """Draw the background color and image of the table children."""
-    for column_group in table.column_groups:
-        draw_background(stream, column_group.background)
-        for column in column_group.children:
-            draw_background(stream, column.background)
-    for row_group in table.children:
-        draw_background(stream, row_group.background)
-        for row in row_group.children:
-            draw_background(stream, row.background)
-            for cell in row.children:
-                if (table.style['border_collapse'] == 'collapse' or
-                        cell.style['empty_cells'] == 'show' or
-                        not cell.empty):
-                    draw_background(stream, cell.background)
-
-
 def draw_background_image(stream, layer, image_rendering):
     if layer.image is None or 0 in layer.size:
         return
@@ -391,7 +361,7 @@ def draw_background_image(stream, layer, image_rendering):
             position_x = 0
         else:
             # We don't repeat the image.
-            repeat_width = image_width
+            repeat_width = positioning_width
 
     # Comments above apply here too.
     if repeat_y == 'no-repeat':
@@ -406,7 +376,7 @@ def draw_background_image(stream, layer, image_rendering):
                 positioning_height - image_height) / (n_repeats - 1)
             position_y = 0
         else:
-            repeat_height = image_height
+            repeat_height = positioning_height
 
     matrix = Matrix(e=position_x + positioning_x, f=position_y + positioning_y)
     matrix @= stream.ctm
@@ -428,11 +398,6 @@ def draw_background_image(stream, layer, image_rendering):
         stream.fill()
 
 
-def xy_offset(x, y, offset_x, offset_y, offset):
-    """Increment X and Y coordinates by the given offsets."""
-    return x + offset_x * offset, y + offset_y * offset
-
-
 def styled_color(style, color, side):
     if style in ('inset', 'outset'):
         do_lighten = (side in ('top', 'left')) ^ (style == 'inset')
@@ -775,6 +740,34 @@ def draw_outlines(stream, box):
                 draw_outlines(stream, child)
 
 
+def draw_table(stream, table):
+    # Draw backgrounds
+    draw_background(stream, table.background)
+    for column_group in table.column_groups:
+        draw_background(stream, column_group.background)
+        for column in column_group.children:
+            draw_background(stream, column.background)
+    for row_group in table.children:
+        draw_background(stream, row_group.background)
+        for row in row_group.children:
+            draw_background(stream, row.background)
+            for cell in row.children:
+                if (table.style['border_collapse'] == 'collapse' or
+                        cell.style['empty_cells'] == 'show' or
+                        not cell.empty):
+                    draw_background(stream, cell.background)
+
+    # Draw borders
+    if table.style['border_collapse'] == 'collapse':
+        return draw_collapsed_borders(stream, table)
+    draw_border(stream, table)
+    for row_group in table.children:
+        for row in row_group.children:
+            for cell in row.children:
+                if cell.style['empty_cells'] == 'show' or not cell.empty:
+                    draw_border(stream, cell)
+
+
 def draw_collapsed_borders(stream, table):
     """Draw borders of table cells when they collapse."""
     row_heights = [
@@ -1083,7 +1076,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y,
     while runs[-1].next != ffi.NULL:
         runs.append(runs[-1].next)
 
-    matrix = Matrix(font_size, 0, 0, -font_size, x, y)
+    matrix = Matrix(1, 0, 0, -1, x, y)
     if angle:
         matrix = Matrix(a=cos(angle), b=-sin(angle),
                         c=sin(angle), d=cos(angle)) @ matrix
@@ -1115,7 +1108,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y,
                 stream.show_text(string)
             string = ''
             last_font = font
-        stream.set_font_size(font.hash, 1)
+        stream.set_font_size(font.hash, 1 if font.bitmap else font_size)
         string += '<'
         for i in range(num_glyphs):
             glyph_info = glyphs[i]
@@ -1130,7 +1123,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y,
             offset = glyph_info.geometry.x_offset / font_size
             if offset:
                 string += f'>{-offset}<'
-            string += f'{glyph:04x}'
+            string += f'{glyph:02x}' if font.bitmap else f'{glyph:04x}'
 
             # Ink bounding box and logical widths in font
             if glyph not in font.widths:
@@ -1149,9 +1142,9 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y,
                     font.bbox[2] = int(units_to_double(x2 * 1000) / font_size)
                 if y2 > font.bbox[3]:
                     font.bbox[3] = int(units_to_double(y2 * 1000) / font_size)
-                font.widths[glyph] = int(
+                font.widths[glyph] = int(round(
                     units_to_double(stream.logical_rect.width * 1000) /
-                    font_size)
+                    font_size))
 
             # Kerning, word spacing, letter spacing
             kerning = int(
