@@ -10,7 +10,7 @@ import unicodedata
 import tinycss2.color3
 
 from .. import html
-from ..css import computed_values, properties
+from ..css import computed_values, properties, targets
 from ..logger import LOGGER
 from . import boxes
 
@@ -44,7 +44,7 @@ ASCII_TO_WIDE.update({0x20: '\u3000', 0x2D: '\u2212'})
 
 LINE_FEED_RE = re.compile('\r\n?')
 TAB_RE = re.compile('[\t ]*\n[\t ]*')
-SPACE_RE = re.compile(' +')
+SPACE_RE = re.compile('[\t ]+')
 
 
 def create_anonymous_boxes(box):
@@ -89,9 +89,8 @@ def build_formatting_structure(element_tree, style_for, get_image_from_uri,
 
 
 def make_box(element_tag, style, content, element):
-    box = BOX_TYPE_FROM_DISPLAY[style['display'][:2]](
+    return BOX_TYPE_FROM_DISPLAY[style['display'][:2]](
         element_tag, style, element, content)
-    return box
 
 
 def element_to_box(element, style_for, get_image_from_uri, base_url,
@@ -369,26 +368,6 @@ def marker_to_box(element, state, parent_style, style_for, get_image_from_uri,
     yield marker_box
 
 
-def _collect_missing_counter(counter_name, counter_values, missing_counters):
-    """Collect missing counters."""
-    if counter_name not in list(counter_values) + missing_counters:
-        missing_counters.append(counter_name)
-
-
-def _collect_missing_target_counter(counter_name, lookup_counter_values,
-                                    anchor_name, missing_target_counters):
-    """Collect missing target counters.
-
-    The corresponding TargetLookupItem caches the target's page based
-    counter values during pagination.
-
-    """
-    if counter_name not in lookup_counter_values:
-        missing_counters = missing_target_counters.setdefault(anchor_name, [])
-        if counter_name not in missing_counters:
-            missing_counters.append(counter_name)
-
-
 def compute_content_list(content_list, parent_box, counter_values, css_token,
                          parse_again, target_collector, counter_style,
                          get_image_from_uri=None, quote_depth=None,
@@ -447,29 +426,10 @@ def compute_content_list(content_list, parent_box, counter_values, css_token,
                 content_boxes.append(
                     boxes.InlineReplacedBox.anonymous_from(parent_box, image))
         elif type_ == 'content()':
-            added_text = TEXT_CONTENT_EXTRACTORS[value](parent_box)
+            added_text = extract_text(value, parent_box)
             # Simulate the step of white space processing
             # (normally done during the layout)
             add_text(added_text.strip())
-        elif type_ == 'counter()':
-            counter_name, counter_type = value
-            if need_collect_missing:
-                _collect_missing_counter(
-                    counter_name, counter_values, missing_counters)
-            if counter_type != 'none':
-                counter_value = counter_values.get(counter_name, [0])[-1]
-                add_text(
-                    counter_style.render_value(counter_value, counter_type))
-        elif type_ == 'counters()':
-            counter_name, separator, counter_type = value
-            if need_collect_missing:
-                _collect_missing_counter(
-                    counter_name, counter_values, missing_counters)
-            if counter_type != 'none':
-                add_text(separator.join(
-                    counter_style.render_value(counter_value, counter_type)
-                    for counter_value
-                    in counter_values.get(counter_name, [0])))
         elif type_ == 'string()':
             if not in_page_context:
                 # string() is currently only valid in @page context
@@ -479,54 +439,53 @@ def compute_content_list(content_list, parent_box, counter_values, css_token,
                     ' '.join(value))
                 continue
             add_text(context.get_string_set_for(page, *value))
-        elif type_ == 'target-counter()':
-            anchor_token, counter_name, counter_type = value
-            lookup_target = target_collector.lookup_target(
-                anchor_token, parent_box, css_token, parse_again)
-            if lookup_target.state == 'up-to-date':
-                target_values = lookup_target.target_box.cached_counter_values
-                if need_collect_missing:
-                    _collect_missing_target_counter(
-                        counter_name, target_values,
-                        target_collector.anchor_name_from_token(anchor_token),
-                        missing_target_counters)
-                # Mixin target's cached page counters.
-                # cached_page_counter_values are empty during layout.
-                local_counters = (
-                    lookup_target.cached_page_counter_values.copy())
-                local_counters.update(target_values)
-                if counter_type != 'none':
-                    counter_value = local_counters.get(counter_name, [0])[-1]
-                    add_text(counter_style.render_value(
-                        counter_value, counter_type))
+        elif type_ in ('counter()', 'counters()'):
+            counter_name, counter_type = value[0], value[-1]
+            if counter_type == 'none':
+                continue
+            if need_collect_missing:
+                if counter_name not in list(counter_values) + missing_counters:
+                    missing_counters.append(counter_name)
+            if type_ == 'counter()':
+                counter_value = counter_values.get(counter_name, [0])[-1]
+                text = counter_style.render_value(counter_value, counter_type)
             else:
-                break
-        elif type_ == 'target-counters()':
-            anchor_token, counter_name, separator, counter_type = value
+                separator = value[1]
+                text = separator.join(
+                    counter_style.render_value(counter_value, counter_type)
+                    for counter_value in counter_values.get(counter_name, [0]))
+            add_text(text)
+        elif type_ in ('target-counter()', 'target-counters()'):
+            (anchor_token, counter_name), counter_type = value[:2], value[-1]
+            if counter_type == 'none':
+                continue
             lookup_target = target_collector.lookup_target(
                 anchor_token, parent_box, css_token, parse_again)
-            if lookup_target.state == 'up-to-date':
+            if lookup_target.state != 'up-to-date':
+                break
+            target_values = lookup_target.target_box.cached_counter_values
+            if need_collect_missing and counter_name not in target_values:
+                anchor_name = targets.anchor_name_from_token(anchor_token)
+                missing_counters = missing_target_counters.setdefault(
+                    anchor_name, [])
+                if counter_name not in missing_counters:
+                    missing_counters.append(counter_name)
+            # Mixin target's cached page counters.
+            # cached_page_counter_values are empty during layout.
+            local_counters = lookup_target.cached_page_counter_values.copy()
+            local_counters.update(target_values)
+            if type_ == 'target-counter()':
+                counter_value = local_counters.get(counter_name, [0])[-1]
+                text = counter_style.render_value(counter_value, counter_type)
+            else:
+                separator = value[2]
                 if separator[0] != 'string':
                     break
                 separator_string = separator[1]
-                target_values = lookup_target.target_box.cached_counter_values
-                if need_collect_missing:
-                    _collect_missing_target_counter(
-                        counter_name, target_values,
-                        target_collector.anchor_name_from_token(anchor_token),
-                        missing_target_counters)
-                # Mixin target's cached page counters.
-                # cached_page_counter_values are empty during layout.
-                local_counters = (
-                    lookup_target.cached_page_counter_values.copy())
-                local_counters.update(target_values)
-                if counter_type != 'none':
-                    add_text(separator_string.join(
-                        counter_style.render_value(counter_value, counter_type)
-                        for counter_value
-                        in local_counters.get(counter_name, [0])))
-            else:
-                break
+                text = separator_string.join(
+                    counter_style.render_value(counter_value, counter_type)
+                    for counter_value in local_counters.get(counter_name, [0]))
+            add_text(text)
         elif type_ == 'target-text()':
             anchor_token, text_style = value
             lookup_target = target_collector.lookup_target(
@@ -535,15 +494,13 @@ def compute_content_list(content_list, parent_box, counter_values, css_token,
                 target_box = lookup_target.target_box
                 # TODO: 'before'- and 'after'- content referring missing
                 # counters are not properly set.
-                text = TEXT_CONTENT_EXTRACTORS[text_style](target_box)
+                text = extract_text(text_style, target_box)
                 # Simulate the step of white space processing
                 # (normally done during the layout)
                 add_text(text.strip())
             else:
                 break
-        elif (type_ == 'quote' and
-                quote_depth is not None and
-                quote_style is not None):
+        elif type_ == 'quote' and None not in (quote_depth, quote_style):
             is_open = 'open' in value
             insert = not value.startswith('no-')
             if not is_open:
@@ -575,6 +532,8 @@ def compute_content_list(content_list, parent_box, counter_values, css_token,
                         context=context, page=page)
             content_boxes.append(new_box)
         elif type_ == 'leader()':
+            if not value[1]:
+                continue
             text_box = boxes.TextBox.anonymous_from(parent_box, value[1])
             leader_box = boxes.InlineBox.anonymous_from(
                 parent_box, (text_box,))
@@ -828,8 +787,12 @@ def table_boxes_children(box, children):
         # Rule XXX (not in the spec): column groups have at least
         # one column child.
         if not children:
+            if box.span is None or box.span < 1:
+                span = 1
+            else:
+                span = box.span
             children = [boxes.TableColumnBox.anonymous_from(box, [])
-                        for _i in range(box.span)]
+                        for _ in range(span)]
 
     # rule 1.3
     if box.tabular_container and len(children) >= 2:
@@ -947,7 +910,6 @@ def wrap_table(box, children):
                 # already generates x TableColumnBox children
                 column.grid_x = grid_x
                 grid_x += 1
-            group.span = len(group.children)
         else:
             grid_x += group.span
     grid_width = grid_x
@@ -1260,9 +1222,7 @@ def process_whitespace(box, following_collapsible_space=False):
             text = text.replace('\n', ' ')
 
         if space_collapse:
-            text = text.replace('\t', ' ')
-            text = SPACE_RE.sub(' ', text)
-            previous_text = text
+            previous_text = text = SPACE_RE.sub(' ', text)
             if following_collapsible_space and text.startswith(' '):
                 text = text[1:]
                 box.leading_collapsible_space = True
@@ -1604,50 +1564,29 @@ def box_text(box):
             not child.element_tag.endswith('::after') and
             not child.element_tag.endswith('::marker') and
             isinstance(child, boxes.TextBox))
-    else:
+    return ''
+
+
+def extract_text(text_part, box):
+    if text_part in ('text', 'content'):
+        return box_text(box)
+    elif text_part in ('before', 'after'):
+        if isinstance(box, boxes.ParentBox):
+            return ''.join(
+                box_text(child) for child in box.descendants()
+                if child.element_tag.endswith(f'::{text_part}') and
+                not isinstance(child, boxes.ParentBox))
         return ''
-
-
-def box_text_first_letter(box):
-    # TODO: use the same code as in inlines.first_letter_to_box
-    character_found = False
-    first_letter = ''
-    text = box_text(box)
-    while text:
-        next_letter = text[0]
-        category = unicodedata.category(next_letter)
-        if category not in ('Ps', 'Pe', 'Pi', 'Pf', 'Po'):
-            if character_found:
-                break
-            character_found = True
-        first_letter += next_letter
-        text = text[1:]
-    return first_letter
-
-
-def box_text_before(box):
-    if isinstance(box, boxes.ParentBox):
-        return ''.join(
-            box_text(child) for child in box.descendants()
-            if child.element_tag.endswith('::before') and
-            not isinstance(child, boxes.ParentBox))
-    else:
-        return ''
-
-
-def box_text_after(box):
-    if isinstance(box, boxes.ParentBox):
-        return ''.join(
-            box_text(child) for child in box.descendants()
-            if child.element_tag.endswith('::after') and
-            not isinstance(child, boxes.ParentBox))
-    else:
-        return ''
-
-
-TEXT_CONTENT_EXTRACTORS = {
-    'text': box_text,
-    'content': box_text,
-    'before': box_text_before,
-    'after': box_text_after,
-    'first-letter': box_text_first_letter}
+    elif text_part == 'first-letter':
+        # TODO: use the same code as in inlines.first_letter_to_box
+        character_found = False
+        first_letter = ''
+        text = box_text(box)
+        for letter in text:
+            category = unicodedata.category(letter)
+            if category not in ('Ps', 'Pe', 'Pi', 'Pf', 'Po'):
+                if character_found:
+                    break
+                character_found = True
+            first_letter += letter
+        return first_letter
