@@ -327,7 +327,7 @@ class LayoutContext:
         """Add a footnote to the layout for this page."""
         self.footnotes.remove(footnote)
         self.current_page_footnotes.append(footnote)
-        self._update_footnote_area()
+        return self._update_footnote_area()
 
     def unlayout_footnote(self, footnote):
         """Remove a footnote from the layout and return it to the waitlist."""
@@ -356,10 +356,16 @@ class LayoutContext:
         if self.current_footnote_area.children:
             footnote_area = build.create_anonymous_boxes(
                 self.current_footnote_area.deepcopy())
-            footnote_area, _, _, _, _ = block_level_layout(
+            footnote_area, _, _, _, _, _ = block_level_layout(
                 self, footnote_area, -inf, None,
-                self.current_footnote_area.page, True, [], [], [], False)
+                self.current_footnote_area.page, True, [], [], [], False, None)
             self.current_footnote_area.height = footnote_area.height
             self.page_bottom -= footnote_area.margin_height()
+            last_child = footnote_area.children[-1]
+            overflow = (
+                last_child.position_y + last_child.margin_height() >
+                footnote_area.position_y + footnote_area.margin_height())
+            return overflow
         else:
             self.current_footnote_area.height = 0
+            return False
